import ApplicationComponent from "../ApplicationComponent";
import {NextFunction, Request, Response, Router} from "express";
import AuthGuard from "./AuthGuard";
import Controller from "../Controller";
import {ForbiddenHttpError} from "../HttpError";

export default class AuthComponent extends ApplicationComponent<void> {
    private readonly authGuard: AuthGuard<any>;

    public constructor(authGuard: AuthGuard<any>) {
        super();
        this.authGuard = authGuard;
    }

    public async init(router: Router): Promise<void> {
        router.use(async (req, res, next) => {
            req.authGuard = this.authGuard;
            req.models.user = res.locals.user = await (await req.authGuard.getProof(req))?.getResource();
            next();
        });
    }
}

export const REQUIRE_REQUEST_AUTH_MIDDLEWARE = async (req: Request, res: Response, next: NextFunction): Promise<void> => {
    let proof = await req.authGuard.isAuthenticatedViaRequest(req);
    if (!proof) {
        req.flash('error', `You must be logged in to access ${req.url}.`);
        res.redirect(Controller.route('auth', undefined, {
            redirect_uri: req.url,
        }));
        return;
    }

    next();
};

export const REQUIRE_AUTH_MIDDLEWARE = async (req: Request, res: Response, next: NextFunction): Promise<void> => {
    // Via request
    let proof = await req.authGuard.isAuthenticatedViaRequest(req);
    if (proof) {
        next();
        return;
    }

    // Via session
    proof = await req.authGuard.isAuthenticated(req.session!);
    if (!proof) {
        req.flash('error', `You must be logged in to access ${req.url}.`);
        res.redirect(Controller.route('auth', undefined, {
            redirect_uri: req.url,
        }));
        return;
    }

    next();
};
export const REQUIRE_GUEST_MIDDLEWARE = async (req: Request, res: Response, next: NextFunction): Promise<void> => {
    if (await req.authGuard.isAuthenticated(req.session!)) {
        res.redirectBack();
        return;
    }

    next();
};
export const REQUIRE_ADMIN_MIDDLEWARE = async (req: Request, res: Response, next: NextFunction): Promise<void> => {
    if (!req.models.user || !req.models.user.is_admin) {
        throw new ForbiddenHttpError('secret tool', req.url);
    }

    next();
};