import Migration from "../../db/Migration";
import {Connection} from "mysql";
import ModelFactory from "../../db/ModelFactory";
import User from "../models/User";
import UserEmail from "../models/UserEmail";

export default class CreateUsersAndUserEmailsTable extends Migration {
    public async install(connection: Connection): Promise<void> {
        await this.query(`CREATE TABLE users
            (
                id         INT      NOT NULL AUTO_INCREMENT,
                name       VARCHAR(64),
                is_admin   BOOLEAN  NOT NULL DEFAULT false,
                created_at DATETIME NOT NULL DEFAULT NOW(),
                updated_at DATETIME NOT NULL DEFAULT NOW(),
                PRIMARY KEY (id)
            )`, connection);
        await this.query(`CREATE TABLE user_emails
            (
                id         INT                 NOT NULL AUTO_INCREMENT,
                user_id    INT                 NOT NULL,
                email      VARCHAR(254) UNIQUE NOT NULL,
                main       BOOLEAN             NOT NULL,
                created_at DATETIME            NOT NULL DEFAULT NOW(),
                PRIMARY KEY (id),
                FOREIGN KEY user_fk (user_id) REFERENCES users (id) ON DELETE CASCADE
            )`, connection);
    }

    public async rollback(connection: Connection): Promise<void> {
        await this.query('DROP TABLE user_emails', connection);
        await this.query('DROP TABLE users', connection);
    }

    public registerModels(): void {
        ModelFactory.register(User);
        ModelFactory.register(UserEmail);
    }
}
